@@ -23,6 +23,7 @@ from optparse import OptionGroup, OptionParser
 import os
 import platform
 import re
+import string
 import sys
 import time
 import threading
@@ -437,11 +438,13 @@ class Download(threading.Thread):
     """
     Start = False
 
-    def __init__(self, url, filename, group=None,
+    def __init__(self, id, url, filename, group=None,
                     target=None, name=None, verbose=None):
         """
         Constructor.
 
+        @type   id:     string
+        @param  id:     value of <guid>, a permanent link identifying the url
         @type   url:    string
         @param  url:    file url to download
         @type   filename:   string
@@ -456,6 +459,7 @@ class Download(threading.Thread):
         @type   verbose:
         @param  verbose:
         """
+        self._id = id
         self._url = url
         self._filename = filename
         self.is_ended = False
@@ -482,7 +486,6 @@ class Download(threading.Thread):
             else:
                 self.size = int(self.size)
             local_file = open(self._filename + '.part', 'wb')
-            #local_file.write(h.read())
             chunk_size = 8192
             while 1:
                 chunk = url_handler.read(chunk_size)
@@ -502,12 +505,16 @@ class Download(threading.Thread):
                 os.rename(self._filename + '.part', self._filename)
                 self.is_ended = True
         except urllib.error.HTTPError as error:
+            self.size = 1
             self.log('HTTP Error: ' + self._url + " " + \
                         HTTP_RESPONSES[error.code][0])
         except urllib.error.URLError as error:
+            self.size = 1
             self.log("URL Error: %s %s" % (self._url, error.reason))
         except IOError as error:
+            self.size = 1
             self.log("IOError : %s %s" % (self._url, error.strerror))
+        self.is_ended = True
 
     def stop(self):
         """
@@ -524,6 +531,15 @@ class Download(threading.Thread):
         """
         print("IIII %s" % text)
 
+    def get_id(self):
+        """
+        Accessor to the ID (<guid>) of the URL.
+
+        @rtype:     string
+        @return:    URL/id
+        """
+        return self._id
+
     def get_url(self):
         """
         Accessor to the URL to download.
@@ -545,6 +561,7 @@ class FeedReader(threading.Thread):
     - the enclosures from entry/link[rel=enclosure]
     """
     _info = {
+        "id": None,
         "url": None,
         "pubDate": None,
         "title": None,
@@ -602,6 +619,9 @@ class FeedReader(threading.Thread):
                     ):
                     if attr in item.attrib:
                         info[info_attr] = item.attrib[attr]
+                # guid
+                info["id"] = info["url"]
+                #
                 result["files"].append(info)
         return result
 
@@ -624,6 +644,13 @@ class FeedReader(threading.Thread):
             for attr in ("url", "type", "length"):
                 if attr in enclosure.attrib:
                     info[attr] = enclosure.attrib[attr]
+            # guid
+            info["id"] = info["url"]
+            guid = item.find("guid")
+            if guid != None:
+                if ("isPermaLink" in guid.attrib and guid.attrib["isPermaLink"] != "false") or not "isPermaLink" in guid.attrib:
+                    info["id"] = guid.text
+            #
             result["files"].append(info)
         return result
 
@@ -638,8 +665,8 @@ class FeedReader(threading.Thread):
                 return
             self.title = results["title"]
             for info in results["files"]:
-                self._urls.append((info["url"], info["pubDate"],
-                                    info["title"]))
+                self._urls.append((info["id"], info["url"],
+                                info["pubDate"], info["title"]))
         except urllib.error.HTTPError as error:
             self.log("HTTP Error: %s %s" %
                     (self._url, HTTP_RESPONSES[error.code][0]))
@@ -703,13 +730,13 @@ class Elatom(threading.Thread):
                 if feed_reader.is_ended and not name in urls:
                     urls[name] = []
                     file_nb = 0
-                    for (url, pubdate, title) in feed_reader.get_urls():
+                    for (id, url, pubdate, title) in feed_reader.get_urls():
                         # clean the URL
                         url = self._clean_url(url)
                         # if downloaded, ignore
-                        if self._config.is_downloaded(url):
+                        if self._config.is_downloaded(id):
                             continue
-                        urls[name].append((url, pubdate, title))
+                        urls[name].append((id, url, pubdate, title))
                         file_nb += 1
                     self._app.update_feed_info(name,
                                     n_("%d file", "%d files", file_nb) % file_nb, 0, file_nb)
@@ -743,7 +770,7 @@ class Elatom(threading.Thread):
             feed_title = feed_readers[name].title
             # clean the title
             feed_title = Elatom.format_title(feed_title)
-            for (url, pubdate, title) in urls[name]:
+            for (id, url, pubdate, title) in urls[name]:
                 # format the date
                 pubdate = Elatom.format_date(pubdate)
                 # clean the title
@@ -755,7 +782,7 @@ class Elatom(threading.Thread):
                                 pubdate + '_' + feed_title + "_" + title + ext)
                 # download,
                 # and set a progress bar on each feed for file progress
-                download_thread = Download(url, filename)
+                download_thread = Download(id, url, filename)
                 download_threads[name].append(download_thread)
                 download_thread.start()
                 total += 1
@@ -795,7 +822,7 @@ class Elatom(threading.Thread):
                             i = len(downloaded_urls)
                         downloaded[name] += download_thread.size
                         # save
-                        self._config.memory(download_thread.get_url())
+                        self._config.memory(download_thread.get_id())
                     elif download_thread.downloaded != None:
                         downloaded[name] += download_thread.downloaded
                 if sizes[name]["total"] == downloaded[name]:
@@ -883,10 +910,12 @@ class Elatom(threading.Thread):
         """
         pattern = re.compile('\W')
         title = re.sub(pattern, '', title)
-        return title.replace('FranceInter', '')\
+        title = title.replace('FranceInter', '')\
                     .replace('FranceCulture', '')\
                     .replace('Podcast', '')\
-                    .replace('podcast', '').strip()[0:20]
+                    .replace('podcast', '').strip()
+        return ''.join([x for x in title if x in string.printable])[0:20]
+
 
 ################
 #              #
